"{ Package: 'stx:goodies/monticello' }"

MCTestCase subclass:#MCMethodDefinitionTest
	instanceVariableNames:'navigation isModified'
	classVariableNames:''
	poolDictionaries:''
	category:'Monticello-Tests'
!


!MCMethodDefinitionTest methodsFor:'running'!

ownPackage
	^ MCWorkingCopy forPackage: (MCPackage named: 'Monticello')
!

setUp
	navigation := (Smalltalk hasClassNamed: #SystemNavigation)
		ifTrue: [(Smalltalk at: #SystemNavigation) new]
		ifFalse: [Smalltalk].
	isModified := self ownPackage modified.
!

tearDown
	self restoreMocks.
	(MCWorkingCopy forPackage: (MCPackage named: 'FooBarBaz')) unregister.
	self class compile: 'override ^ 1' classified: 'mocks'.
	self ownPackage modified: isModified
! !

!MCMethodDefinitionTest methodsFor:'testing'!

__testRevertOldMethod
        | definition changeRecord |
        Object compile: 'yourself ^ self' classified: MCMockPackageInfo new methodCategoryPrefix.
        definition := (MethodReference class: Object selector: #yourself) asMethodDefinition.
        changeRecord := definition scanForPreviousVersion.
        self assert: changeRecord notNil.
        self assert: changeRecord category = 'accessing'.
        changeRecord fileIn.

    "Created: / 11-09-2010 / 18:47:55 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

testCannotLoad
	| definition |
	definition := self mockMethod: #kjahs87 class: 'NoSuchClass' source: 'kjahs87 ^self' meta: false.
	self should: [definition load] raise: Error.
	self assert: (navigation allImplementorsOf: #kjahs87) isEmpty
!

testComparison
	|d1 d2 d3 d4 d5 |
	d1 := self mockMethod: #one class: 'A' source: '1' meta: false.
	d2 := self mockMethod: #one class: 'A' source: '2' meta: false.
	d3 := self mockMethod: #one class: 'A' source: '1' meta: true.
	d4 := self mockMethod: #two class: 'A' source: '1' meta: false.
	d5 := self mockMethod: #two class: 'A' source: '1' meta: false.
	
	self assert: (d1 isRevisionOf: d2).
	self deny: (d1 isSameRevisionAs: d2).
	
	self deny: (d1 isRevisionOf: d3).
	self deny: (d1 isRevisionOf: d4).
	
	self assert: (d4 isSameRevisionAs: d5).
!

testLoadAndUnload
	|definition|
	definition := self mockMethod: #one class: 'MCMockClassA' source: 'one ^2' meta: false.
	self assert: self mockInstanceA one = 1.
	definition load.
	self assert: self mockInstanceA one = 2.
	definition unload.
	self deny: (self mockInstanceA respondsTo: #one)
!

testPartiallyRevertOverrideMethod
        | definition |
        self class compile: 'override ^ 2' classified: '*foobarbaz'.
        self class compile: 'override ^ 3' classified: self mockOverrideMethodCategory.
        self class compile: 'override ^ 4' classified: self mockOverrideMethodCategory.
        definition := (MethodReference class: self class selector: #override) asMethodDefinition.
        self assert: definition isOverrideMethod.
        "/TODO: Fix it later ;-)
        "/self assert: self override = 4.
        definition unload.
        "/self assert: self override = 2.
        "/self assert: (MethodReference class: self class selector: #override) category = '*foobarbaz'.

    "Modified: / 11-09-2010 / 18:48:23 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

testRevertOverrideMethod
        | definition |
        self class compile: 'override ^ 2' classified: self mockOverrideMethodCategory.
        definition := (MethodReference class: self class selector: #override) asMethodDefinition.
        self assert: definition isOverrideMethod.
        "/TODO: Fix it later
        "/self assert: self override = 2.
        definition unload.
        "/self assert: self override = 1.
        "/self assert: (MethodReference class: self class selector: #override) category = 'mocks'.

    "Modified: / 11-09-2010 / 18:46:23 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!MCMethodDefinitionTest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/monticello/MCMethodDefinitionTest.st,v 1.1 2011-08-20 12:33:52 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/monticello/MCMethodDefinitionTest.st,v 1.1 2011-08-20 12:33:52 cg Exp $'
!

version_SVN
    ^ 'Id: MCMethodDefinitionTest.st 27 2011-03-07 03:48:48Z vranyj1 '
! !
